@@ -169,6 +169,20 @@ add_compress_arg (FRCommand *comm)
 }
 
 
+static void
+begin_tar_command (FRCommand *comm)
+{
+	char *path = NULL;
+
+	path = g_find_program_in_path ("gtar");
+	if (path != NULL)
+		fr_process_begin_command (comm->process, "gtar");
+	else
+		fr_process_begin_command (comm->process, "tar");
+	g_free (path);
+}
+
+
 static void
 fr_command_tar_list (FRCommand *comm)
 {
@@ -176,7 +190,7 @@ fr_command_tar_list (FRCommand *comm)
 				      process_line,
 				      comm);
 
-	fr_process_begin_command (comm->process, "tar");
+	begin_tar_command (comm);
 	fr_process_add_arg (comm->process, "--force-local");
 	fr_process_add_arg (comm->process, "-tvf");
 	fr_process_add_arg (comm->process, comm->e_filename);
@@ -234,7 +248,7 @@ fr_command_tar_add (FRCommand     *comm,
 				      process_line__add,
 				      comm);
 
-	fr_process_begin_command (comm->process, "tar");
+	begin_tar_command (comm);
 	fr_process_add_arg (comm->process, "--force-local");
 	fr_process_add_arg (comm->process, "-v");
 
@@ -285,7 +299,7 @@ fr_command_tar_delete (FRCommand *comm,
 				      process_line__delete,
 				      comm);
 
-	fr_process_begin_command (comm->process, "tar");
+	begin_tar_command (comm);
 	fr_process_set_begin_func (comm->process, begin_func__delete, comm);
 	fr_process_add_arg (comm->process, "--force-local");
 	fr_process_add_arg (comm->process, "-v");
@@ -322,7 +336,7 @@ fr_command_tar_extract (FRCommand  *comm,
 				      process_line__extract,
 				      comm);
 
-	fr_process_begin_command (comm->process, "tar");
+	begin_tar_command (comm);
 	fr_process_add_arg (comm->process, "--force-local");
 	fr_process_add_arg (comm->process, "-v");
 	fr_process_add_arg (comm->process, "-xf");
@@ -607,14 +621,12 @@ fr_command_tar_uncompress (FRCommand *comm)
 	if (c_tar->name_modified) {
 		tmp_name = get_temp_name (c_tar, comm->filename);
 		if (path_is_file (comm->filename)) {
-			g_print ("test %s [OK]\n", comm->filename);
 			fr_process_begin_command (comm->process, "mv");
 			fr_process_add_arg (comm->process, "-f");
 			fr_process_add_arg (comm->process, comm->e_filename);
 			fr_process_add_arg (comm->process, tmp_name);
 			fr_process_end_command (comm->process);
-		} else
-			g_print ("test %s [NO]\n", comm->filename);
+		}
 	} else
 		tmp_name = g_strdup (comm->e_filename);
 
@@ -741,7 +753,7 @@ fr_command_tar_finalize (GObject *object)
 GType
 fr_command_tar_get_type ()
 {
-        static guint type = 0;
+        static GType type = 0;
 
         if (! type) {
                 GTypeInfo type_info = {
