@@ -37,17 +37,6 @@
 #include "menu-callbacks.h"
 
 
-void
-new_window_cb (GtkWidget *widget, 
-	       void      *data)
-{
-	FRWindow *window;
-
-	window = window_new ();
-	gtk_widget_show (window->app);
-}
-
-
 void
 close_window_cb (GtkWidget *widget, 
 		 void      *data)
@@ -71,8 +60,21 @@ new_archive (GtkWidget *file_sel,
 	     FRWindow  *window, 
 	     gchar     *path)
 {
-	window_archive_new (window, path);	
-	gtk_widget_destroy (file_sel);
+	FRWindow *archive_window;
+	gboolean  new_window;
+
+	new_window = window->archive_present;
+
+	if (new_window) 
+		archive_window = window_new ();
+	else
+		archive_window = window;
+
+	if (window_archive_new (archive_window, path)) {
+		gtk_window_present (GTK_WINDOW (archive_window->app));
+		gtk_widget_destroy (file_sel);
+	} else if (new_window)
+		window_close (archive_window);
 }
 
 
@@ -104,7 +106,7 @@ typedef struct {
 } FileTypeDescription;
 
 
-FileTypeDescription type_desc[] = { 
+FileTypeDescription write_type_desc[] = { 
 	{ N_("Automatic"),                             NULL},
 	{ N_("Arj (.arj)"),                            ".arj" },
 	{ N_("Ear (.ear)"),                            ".ear" },
@@ -122,48 +124,56 @@ FileTypeDescription type_desc[] = {
 	{ N_("Zoo (.zoo)"),                            ".zoo" }
 };
 
-static int type_desc_n = sizeof (type_desc) / sizeof (FileTypeDescription);
-
 
 static char *
 get_full_path (GtkWidget *file_sel)
 {
 	GtkWidget   *opt_menu;
 	char        *full_path;
-	const char  *path;
+	char        *path;
 	const char  *filename;
 	int          file_type_idx;
 
 	opt_menu = g_object_get_data (G_OBJECT (file_sel), "fr_opt_menu");
-	path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel));
+	path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel));
 
 	if ((path == NULL) || (*path == 0))
 		return NULL;
 
 	filename = file_name_from_path (path);
-	if ((filename == NULL) || (*filename == 0))
+	if ((filename == NULL) || (*filename == 0)) {
+		g_free (path);
 		return NULL;
+	}
 	
 	file_type_idx = opt_menu_get_active_idx (opt_menu);
-	if (file_type_idx > 0) 
+	if (file_type_idx > 0) {
 		full_path = g_strconcat (path, 
-					 type_desc[file_type_idx].ext,
+					 write_type_desc[file_type_idx].ext,
 					 NULL);
-	else
-		full_path = g_strdup (path);
+		g_free (path);
+	} else {
+		full_path = path;
+	}
 
 	return full_path;
 }
 
 
 static void 
-new_file_ok_cb (GtkWidget *w,
-		GtkWidget *file_sel)
+new_file_response_cb (GtkWidget *w,
+		      gint response,
+		      GtkWidget *file_sel)
 {
 	FRWindow *window;
 	gchar    *path;
 	gchar    *dir;
 
+	if (response == GTK_RESPONSE_CANCEL) {
+		gtk_widget_destroy (file_sel);
+		return;
+	}
+
 	window = g_object_get_data (G_OBJECT (file_sel), "fr_window");
 
 	path = get_full_path (file_sel);
@@ -178,7 +188,7 @@ new_file_ok_cb (GtkWidget *w,
 						  GTK_STOCK_DIALOG_ERROR,
 						  _("Could not create the archive"),
 						  _("You have to specify an archive name."),
-						  GTK_STOCK_OK, GTK_RESPONSE_OK,
+						  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 						  NULL);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -197,7 +207,7 @@ new_file_ok_cb (GtkWidget *w,
 						  GTK_STOCK_DIALOG_ERROR,
 						  _("Could not create the archive"),
 						  _("You don't have permission to create an archive in this folder"),
-						  GTK_STOCK_OK, GTK_RESPONSE_OK,
+						  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 						  NULL);
 
 		gtk_dialog_run (GTK_DIALOG (dialog));
@@ -246,7 +256,7 @@ new_file_ok_cb (GtkWidget *w,
 							  GTK_STOCK_DIALOG_ERROR,
 							  _("Could not delete the old archive."),
 							  NULL,
-							  GTK_STOCK_OK, GTK_RESPONSE_OK,
+							  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 							  NULL);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -267,8 +277,8 @@ build_file_type_menu (FRWindow *window)
 	int        i;
 
         menu = gtk_menu_new ();
-        for (i = 0; i < type_desc_n; i++) {
-                item = gtk_menu_item_new_with_label (_(type_desc[i].name));
+        for (i = 0; i < G_N_ELEMENTS (write_type_desc); i++) {
+                item = gtk_menu_item_new_with_label (_(write_type_desc[i].name));
 		gtk_widget_show (item);
                 gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
         }
@@ -281,22 +291,58 @@ void
 new_archive_cb (GtkWidget *widget, 
 		void      *data)
 {
+	static const char * save_mime_type[] = {
+		"application/x-tar",
+		"application/x-compressed-tar",
+		"application/x-bzip-compressed-tar",
+		"application/x-arj",
+		"application/zip",
+		"application/x-java-archive",
+		"application/x-jar",
+		"application/x-lha",
+		"application/x-rar",
+		"application/x-rar-compressed",
+		"application/x-gzip",
+		"application/x-bzip",
+		"application/x-compress",
+		"application/x-lzop",
+		"application/x-zoo",
+	};
 	FRWindow  *window = data;
 	GtkWidget *file_sel;
 	GtkWidget *hbox;
 	GtkWidget *vbox, *vbox2;
 	GtkWidget *opt_menu;
 	GtkWidget *menu;
-	char      *dir;
+	GtkFileFilter *filter;
+	int i;
 
-	file_sel = gtk_file_selection_new (_("New Archive"));
+	file_sel = gtk_file_chooser_dialog_new (_("New"),
+					        NULL,
+					        GTK_FILE_CHOOSER_ACTION_SAVE,
+					        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					        GTK_STOCK_NEW, GTK_RESPONSE_OK,
+					        NULL);
 
-	dir = g_strconcat (window->open_default_dir, "/", NULL);
-	gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_sel), dir);
-	g_free (dir);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_sel),
+					     window->open_default_dir);
+
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, _("All archives"));
+	for (i = 0; i < G_N_ELEMENTS (save_mime_type); i++)
+		gtk_file_filter_add_mime_type (filter, save_mime_type[i]);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_sel), filter);
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_sel), filter);
+
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, _("All files"));
+	gtk_file_filter_add_pattern (filter, "*");
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_sel), filter);
+
+	/**/
 
 	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (file_sel)->action_area), vbox, TRUE, TRUE, 0);
+	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (file_sel), vbox);
 
 	vbox2 = gtk_vbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (vbox), vbox2, TRUE, TRUE, 6);
@@ -320,17 +366,13 @@ new_archive_cb (GtkWidget *widget,
 
 	g_object_set_data (G_OBJECT (file_sel), "fr_window", window);
 	g_object_set_data (G_OBJECT (file_sel), "fr_opt_menu", opt_menu);
+	g_object_set_data (G_OBJECT (window->app), "fr_file_sel", file_sel);
 	
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-			  "clicked", 
-			  G_CALLBACK (new_file_ok_cb), 
+	g_signal_connect (G_OBJECT (file_sel),
+			  "response", 
+			  G_CALLBACK (new_file_response_cb), 
 			  file_sel);
 
-	g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-				  "clicked", 
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (file_sel));
-
 	g_signal_connect (G_OBJECT (file_sel),
 			  "destroy", 
 			  G_CALLBACK (new_file_destroy_cb),
@@ -352,13 +394,19 @@ save_file_destroy_cb (GtkWidget *w,
 
 
 static void 
-save_file_ok_cb (GtkWidget *w,
-		 GtkWidget *file_sel)
+save_file_response_cb (GtkWidget *w,
+		       gint response,
+		       GtkWidget *file_sel)
 {
 	FRWindow *window;
 	char     *path;
 	char     *dir;
 
+	if (response == GTK_RESPONSE_CANCEL) {
+		gtk_widget_destroy (file_sel);
+		return;
+	}
+
 	window = g_object_get_data (G_OBJECT (file_sel), "fr_window");
 
 	path = get_full_path (file_sel);
@@ -373,7 +421,7 @@ save_file_ok_cb (GtkWidget *w,
 						  GTK_STOCK_DIALOG_ERROR,
 						  _("Could not save the archive"),
 						  _("You have to specify an archive name."),
-						  GTK_STOCK_OK, GTK_RESPONSE_OK,
+						  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 						  NULL);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -392,7 +440,7 @@ save_file_ok_cb (GtkWidget *w,
 						  GTK_STOCK_DIALOG_ERROR,
 						  _("Could not save the archive"),
 						  _("You don't have permission to create an archive in this folder"),
-						  GTK_STOCK_OK, GTK_RESPONSE_OK,
+						  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 						  NULL);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -432,7 +480,7 @@ save_file_ok_cb (GtkWidget *w,
 							  GTK_STOCK_DIALOG_ERROR,
 							  _("Could not delete the old archive."),
 							  NULL,
-							  GTK_STOCK_OK, GTK_RESPONSE_OK,
+							  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 							  NULL);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -450,49 +498,76 @@ void
 save_as_archive_cb (GtkWidget *widget, 
 		    void *data)
 {
+	static const char * save_mime_type[] = {
+		"application/x-tar",
+		"application/x-compressed-tar",
+		"application/x-bzip-compressed-tar",
+		"application/x-arj",
+		"application/zip",
+		"application/x-java-archive",
+		"application/x-jar",
+		"application/x-lha",
+		"application/x-rar",
+		"application/x-rar-compressed",
+		"application/x-gzip",
+		"application/x-bzip",
+		"application/x-compress",
+		"application/x-lzop",
+		"application/x-zoo",
+	};
 	FRWindow  *window = data;
 	GtkWidget *file_sel;
 	GtkWidget *hbox;
 	GtkWidget *opt_menu;
 	GtkWidget *menu;
-	char      *dir;
+	GtkFileFilter *filter;
+	int i;
 
-	file_sel = gtk_file_selection_new (_("Save Archive"));
+	file_sel = gtk_file_chooser_dialog_new (_("Save"),
+					        NULL,
+					        GTK_FILE_CHOOSER_ACTION_SAVE,
+					        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					        GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+					        NULL);
 
-	dir = g_strconcat (window->open_default_dir, "/", NULL);
-	gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_sel), dir);
-	g_free (dir);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_sel),
+					     window->open_default_dir);
+
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, _("All archives"));
+	for (i = 0; i < G_N_ELEMENTS (save_mime_type); i++)
+		gtk_file_filter_add_mime_type (filter, save_mime_type[i]);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_sel), filter);
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_sel), filter);
+
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, _("All files"));
+	gtk_file_filter_add_pattern (filter, "*");
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_sel), filter);
 
 	/**/
 
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (file_sel)->action_area), hbox, TRUE, TRUE, 0);
-
+	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (hbox), 
 			    gtk_label_new (_("Archive type:")),
 			    FALSE, FALSE, 0);
-
 	opt_menu = gtk_option_menu_new ();
 	menu = build_file_type_menu (window);
         gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_menu), menu);
-	gtk_widget_show (opt_menu);
-	gtk_box_pack_start (GTK_BOX (hbox), opt_menu, FALSE, FALSE, 12);
-	
+	gtk_box_pack_start (GTK_BOX (hbox), opt_menu, TRUE, TRUE, 0);
+	gtk_widget_show_all (hbox);
+
+	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (file_sel), hbox);	
 	/**/
 
 	g_object_set_data (G_OBJECT (file_sel), "fr_window", window);
 	g_object_set_data (G_OBJECT (file_sel), "fr_opt_menu", opt_menu);
 	
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-			  "clicked", 
-			  G_CALLBACK (save_file_ok_cb), 
+	g_signal_connect (G_OBJECT (file_sel),
+			  "response", 
+			  G_CALLBACK (save_file_response_cb), 
 			  file_sel);
 
-	g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-				  "clicked", 
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (file_sel));
-
 	g_signal_connect (G_OBJECT (file_sel),
 			  "destroy", 
 			  G_CALLBACK (save_file_destroy_cb),
@@ -514,20 +589,28 @@ open_file_destroy_cb (GtkWidget *w,
 
 
 static void 
-open_file_ok_cb (GtkWidget *w,
-                 GtkWidget *file_sel)
+open_file_response_cb (GtkWidget *w,
+		       gint response,
+		       GtkWidget *file_sel)
 {
-	FRWindow   *window;
-	const char *path;
+	FRWindow   *window = NULL;
+	char *path;
+
+	if (response == GTK_RESPONSE_CANCEL) {
+		gtk_widget_destroy (file_sel);
+		return;
+	}
 
 	window = g_object_get_data (G_OBJECT (file_sel), "fr_window");
-	path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel));
+	path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel));
 
         if (path == NULL)
                 return;
 
 	if (window_archive_open (window, path, GTK_WINDOW (file_sel)))
 		gtk_widget_destroy (file_sel);
+
+	g_free (path);
 }
 
 
@@ -535,30 +618,61 @@ void
 open_archive_cb (GtkWidget *widget, 
 		 void      *data)
 {
-	GtkWidget *file_sel;
-	FRWindow  *window = data;
-	char      *dir;
+	static const char * open_mime_type[] = {
+		"application/x-tar",
+		"application/x-compressed-tar",
+		"application/x-bzip-compressed-tar",
+		"application/x-arj",
+		"application/zip",
+		"application/x-java-archive",
+		"application/x-jar",
+		"application/x-lha",
+		"application/x-rar",
+		"application/x-rar-compressed",
+		"application/x-rpm",
+		"application/x-stuffit",
+		"application/x-gzip",
+		"application/x-bzip",
+		"application/x-compress",
+		"application/x-lzop",
+		"application/x-zoo",
+	};
+	GtkWidget     *file_sel;
+	FRWindow      *window = data;
+	GtkFileFilter *filter;
+	int            i;
 
-        file_sel = gtk_file_selection_new (_("Open Archive"));
+	file_sel = gtk_file_chooser_dialog_new (_("Open"),
+					        NULL,
+					        GTK_FILE_CHOOSER_ACTION_OPEN,
+					        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					        GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					        NULL);
 
-	dir = g_strconcat (window->open_default_dir, "/", NULL);
-	gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_sel), dir);
-	g_free (dir);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_sel),
+					     window->open_default_dir);
+
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, _("All archives"));
+	for (i = 0; i < G_N_ELEMENTS (open_mime_type); i++)
+		gtk_file_filter_add_mime_type (filter, open_mime_type[i]);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_sel), filter);
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_sel), filter);
+
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, _("All files"));
+	gtk_file_filter_add_pattern (filter, "*");
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_sel), filter);
 
 	/**/
 
 	g_object_set_data (G_OBJECT (file_sel), "fr_window", window);
 	
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-			  "clicked", 
-			  G_CALLBACK (open_file_ok_cb), 
+	g_signal_connect (G_OBJECT (file_sel),
+			  "response", 
+			  G_CALLBACK (open_file_response_cb), 
 			  file_sel);
  
-	g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-				  "clicked", 
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (file_sel));
-
 	g_signal_connect (G_OBJECT (file_sel),
 			  "destroy", 
 			  G_CALLBACK (open_file_destroy_cb),
@@ -597,32 +711,37 @@ file_sel_destroy_cb (GtkWidget    *w,
 
 
 static void 
-copy_or_move_archive_ok_cb (GtkWidget    *w,
-			    CopyMoveData *data)
+copy_or_move_archive_response_cb (GtkWidget    *w,
+				  int           response,
+				  CopyMoveData *data)
 {
-	FRWindow    *window;
-	const char  *path;
-	const char  *filename;
+	FRWindow    *window = data->window;
 	char        *folder;
 	char        *new_path;
 
-	gtk_widget_hide (data->file_sel);
-
-	window = data->window;
-	path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (data->file_sel));
-
-	if ((path == NULL) || (*path == 0)) {
-                gtk_widget_destroy (data->file_sel);
+	if (response == GTK_RESPONSE_CANCEL) {
+		gtk_widget_destroy (w);
 		return;
 	}
 
-	filename = file_name_from_path (path);
-	if ((filename == NULL) || (*filename == 0))
-		filename = file_name_from_path (window->archive_filename);
-	folder = remove_level_from_path (path);
-	new_path = g_build_filename (folder, filename, NULL);
+	gtk_widget_hide (data->file_sel);
+
+	folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (data->file_sel));
+
+	if ((folder == NULL) || (*folder == 0)) {
+		g_free (folder);
+		gtk_widget_show (data->file_sel);
+		return;
+	}
+
+	new_path = g_build_filename (folder, file_name_from_path (window->archive_filename), NULL);
 	g_free (folder);
 
+	if (strcmp (new_path, window->archive_filename) == 0) {
+		gtk_widget_show (data->file_sel);
+		return;
+	}
+
 	if (path_is_file (new_path)) {
 		GtkWidget *d;
 		int        r;
@@ -657,7 +776,7 @@ copy_or_move_archive_ok_cb (GtkWidget    *w,
 							  GTK_STOCK_DIALOG_ERROR,
 							  _("Could not copy the archive"),
 							  NULL,
-							  GTK_STOCK_OK, GTK_RESPONSE_OK,
+							  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 							  NULL);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
@@ -668,6 +787,7 @@ copy_or_move_archive_ok_cb (GtkWidget    *w,
 #endif
 		if (file_move (window->archive_filename, new_path)) 
 			window_archive_rename (window, new_path);
+
 		else {
 			GtkWidget *dialog;
 			dialog = _gtk_message_dialog_new (GTK_WINDOW (window->app),
@@ -675,7 +795,7 @@ copy_or_move_archive_ok_cb (GtkWidget    *w,
 							  GTK_STOCK_DIALOG_ERROR,
 							  _("Could not move the archive"),
 							  NULL,
-							  GTK_STOCK_OK, GTK_RESPONSE_OK,
+							  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 							  NULL);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
@@ -693,32 +813,31 @@ copy_or_move_archive (FRWindow *window,
 		      gboolean  overwrite)
 {
 	GtkWidget    *file_sel;
-	char         *dir;
 	CopyMoveData *data;
 
 	data = g_new (CopyMoveData, 1);
 	data->window = window;
 	data->copy = copy;
 	data->overwrite = overwrite;
-	data->file_sel = file_sel = gtk_file_selection_new (copy ? _("Copy Archive") : _("Move Archive"));
-
-	dir = g_strconcat (window->open_default_dir, "/", NULL);
-	gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_sel), dir);
-	g_free (dir);
-
-	gtk_widget_set_sensitive (GTK_FILE_SELECTION (file_sel)->file_list,
-				  FALSE);
-
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-			  "clicked", 
-			  G_CALLBACK (copy_or_move_archive_ok_cb), 
+	data->file_sel = file_sel = 
+		gtk_file_chooser_dialog_new (copy ? _("Copy") : _("Move"),
+					     NULL,
+					     GTK_FILE_CHOOSER_ACTION_SAVE,
+					     GTK_STOCK_CANCEL,
+					     GTK_RESPONSE_CANCEL,
+					     copy ? _("_Copy"): _("_Move"),
+					     GTK_RESPONSE_OK,
+					     NULL);
+	gtk_file_chooser_set_folder_mode (GTK_FILE_CHOOSER (file_sel),
+					  TRUE);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_sel),
+					     window->open_default_dir);
+ 
+	g_signal_connect (G_OBJECT (file_sel),
+			  "response", 
+			  G_CALLBACK (copy_or_move_archive_response_cb), 
 			  data);
 
-	g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-				  "clicked", 
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (file_sel));
-
 	g_signal_connect (G_OBJECT (file_sel),
 			  "destroy", 
 			  G_CALLBACK (file_sel_destroy_cb),
@@ -778,7 +897,7 @@ rename_archive_cb (GtkWidget *widget,
 	utf8_string = _gtk_request_dialog_run (GTK_WINDOW (window->app),
 					       (GTK_DIALOG_DESTROY_WITH_PARENT 
 						| GTK_DIALOG_MODAL),
-					       _("Rename Archive"),
+					       _("Rename"),
 					       _("New archive name (without extension)"),
 					       utf8_old_string,
 					       1024,
@@ -931,8 +1050,7 @@ void
 quit_cb (GtkWidget *widget, 
 	 void      *data)
 {
-	while (window_list)
-                window_close ((FRWindow*) window_list->data);
+	window_close ((FRWindow*) data);
 }
 
 
@@ -976,6 +1094,30 @@ go_up_one_level_cb (GtkWidget *widget,
 }
 
 
+void
+go_home_cb (GtkWidget *widget, 
+	    void *data)
+{
+	window_go_to_location ((FRWindow*) data, "/");
+}
+
+
+void
+go_back_cb (GtkWidget *widget, 
+	    void *data)
+{
+	window_go_back ((FRWindow*) data);
+}
+
+
+void
+go_forward_cb (GtkWidget *widget, 
+	       void *data)
+{
+	window_go_forward ((FRWindow*) data);
+}
+
+
 void
 set_list_mode_flat_cb (GtkWidget *widget, 
 		       void *data)
@@ -1034,7 +1176,7 @@ manual_cb (GtkWidget *widget,
 						  GTK_STOCK_DIALOG_ERROR,
 						  _("Could not display help"),
 						  err->message,
-						  GTK_STOCK_OK, GTK_RESPONSE_OK,
+						  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 						  NULL);
                 
                 g_signal_connect (G_OBJECT (dialog), "response",
@@ -1243,3 +1385,34 @@ view_statusbar_cb (GtkWidget *widget,
 {
 	eel_gconf_set_boolean (PREF_UI_STATUSBAR, GTK_CHECK_MENU_ITEM (widget)->active);
 }
+
+
+void
+rename_cb (GtkWidget *widget, 
+	   void *data)
+{
+	window_rename_selection ((FRWindow*) data);
+}
+
+
+void
+cut_cb (GtkWidget *widget, 
+	void *data)
+{
+	window_cut_selection ((FRWindow*) data);
+}
+
+
+void
+copy_cb (GtkWidget *widget, 
+	 void *data)
+{
+	window_copy_selection ((FRWindow*) data);
+}
+
+void
+paste_cb (GtkWidget *widget, 
+	  void *data)
+{
+	window_paste_selection ((FRWindow*) data);
+}
